#!/bin/bash
#

# build_mpi4py.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e1[04]>:<p2711|p2713d>:<mpich|mvapich2> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
  [[ -n "${basequal}" ]] || ssi_die "Qualifiers mandatory"

  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  for qual in "${qlist[@]}"; do
    case $qual in
      e1[04])
        cqual=$qual
        ;;
      p2711|p2713d)
        pqual=$qual
        ;;
      mpich|mvapich2)
        bqual=$qual
        ;;
      *)
        ssi_die "Unrecognized qualifier $qual"
    esac
  done
  if [[ -z "$cqual" ]]; then
    ssi_die "Base qualifier is mandatory"
  fi
  if [[ -z "$bqual" ]]; then
    ssi_die "MPI qualifier is mandatory"
  fi
  if [[ -z "$pqual" ]]; then
    ssi_die "A python qualifier is mandatory for e >= 9 (e.g. p2710)."
  fi
  case ${extraqual} in
    prof)
      :
      ;;
    opt|debug)
      ssi_die "${extraqual} builds of numpy are not supported at this time."
      ;;
    *)
      ssi_die "Unrecognized build qualifer ${extraqual}"
  esac
  basequal=${cqual}${pqual:+:${pqual}}${bqual:+:${bqual}}
}

set_compiler_flags()
{
  # We don't compile differently if debug, just use different dependencies.
  cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mpi4py
origpkgver=v2_0_0
pkgver=${origpkgver}d
ssibuildshims_version=v0_17_00
setuptools_version=v22_0_5 # Required only for python < p2713d.
numpy_version=v1_12_1
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims

set --
source ssi_functions

set_qualifiers

source define_basics

set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${blddir} || ssi_die "failed to create ${blddir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "Fake declare of $package failed."

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

(( pv = ${pqual//[A-Za-z]/} ))
if (( ${pv} < 2713 )); then # Bundled with newer python.
  setup -B setuptools ${setuptools_version} -q ${pqual} -z ${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up UPS package for python setuptools ${setuptools_version}"
fi

ups active

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
#   (( MPI4PY_NO_CHECKS = 1 ))
fi

set -x

python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || \
  ssi_die "failed to create ${pkgdir}/lib/python2.7/site-packages"
cd ${blddir} || ssi_die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || ssi_die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || ssi_die "Failed to cd into ${package}-${pkgdotver}"

python setup.py install --prefix ${pkgdir} || ssi_die "python setup.py install failed"
ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

{ (( ncore > 1 )) && \
  [[ -z "${JENKINS_HOME}" ]] && \
  [[ "${bqual}" != mvapich2 ]]; } || (( MPI4PY_NO_CHECKS = 1 ))

if ! (( ${MPI4PY_NO_CHECKS:-0} )); then # Tests no worky on Jenkins or mvapich2
  case $pv in
    2711)
      nose_version=v1_3_7b    
      ;;
    2713)
      nose_version=v1_3_7c
      ;;
    *)
      ssi_die "Unable to ascertain what version of nose to use."
  esac

  setup -B nose $nose_version -q +${pqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up nose ${nose_version} -q +${pqual} for tests."
  setup -B numpy $numpy_version -q +${cqual}:+${pqual}:+${extraqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
    ssi_die "unable to set up numpy ${numpy_version} -q +${cqual}:+${pqual}:+${extraqual} for tests."
  if (( ncore > 5 )); then
    ncore=5
  fi
  (cd test && mpiexec -np ${ncore} -envall nosetests </dev/null || ssi_die "Tests failed.") || exit $?
fi

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "mpi4py is installed at ${MPI4PY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
